/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Header file for the app_test extension. 
 *
 *  This extension provides a way to send configurable messages on the network
 *
 *  See the app_test.conf.sample file for the format of the configuration file.
 */
 
#define IN_EXTENSION
#define DEFINE_DEBUG_MACRO app_test

#include <waaad/waaad.h>

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <assert.h>

/* Mode for the extension */
#define MODE_SERV	0x1
#define	MODE_CLI	0x2

/* The module configuration */
typedef struct {
	uint32_t	vendor_id;	/* default 999999 */
	uint32_t	appli_id;	/* default 123456 */
	uint32_t	cmd_id;		/* default 234567 */
	uint32_t	avp_id;		/* default 345678 */
	int		mode;		/* default MODE_SERV | MODE_CLI */
	char 	*	dest_realm;	/* default g_pconf->realm */
	char 	*	dest_host;	/* default NULL */
	int 		signal;		/* default SIGUSR1 */
} atst_conf_t;
extern atst_conf_t * atst_conf;


/* Parse the configuration file */
int atst_conf_handle(char * conffile);

/* Start or stop the signal handler */
int atst_sig_init(void (*cb)(void));
void atst_sig_fini(void);

/* Handle incoming messages (server) */
int atst_serv_init(void);
void atst_serv_fini(void);

/* Create outgoing message (client) */
int atst_cli_init(void);
void atst_cli_fini(void);

/* Initialize dictionary definitions */
int atst_dict_init(void);


/* Some global variables for dictionary */
extern dict_object_t * atst_vendor;
extern dict_object_t * atst_appli;
extern dict_object_t * atst_cmd_r;
extern dict_object_t * atst_cmd_a;
extern dict_object_t * atst_avp;

extern dict_object_t * atst_sess_id;
extern dict_object_t * atst_origin_host;
extern dict_object_t * atst_origin_realm;
extern dict_object_t * atst_dest_host;
extern dict_object_t * atst_dest_realm;
extern dict_object_t * atst_res_code;

/* Debug level */
extern int app_test_verbosity;

/* Some macro */
#define ASSERT( _x ) assert( _x )

#define CHECK( _call ) {								\
	int __ret = ( _call );								\
	if ( __ret != 0 ) {								\
		TRACE_DEBUG(INFO, "Error in function call");				\
		log_error("Error in extension (" #_call "): %s\n", strerror(__ret));	\
		return __ret;								\
	}										\
}

#define CHECK_VOID( _call ) {								\
	int __ret = ( _call );								\
	if ( __ret != 0 ) {								\
		TRACE_DEBUG(INFO, "Error in function call");				\
		log_error("Error in extension (" #_call "): %s\n", strerror(__ret));	\
		return;								\
	}										\
}
