/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* This file contains compatibility bindings for hostap files.
 Most of the definitions here come from different files from the hostap project.
 
 We don't care for OS-specific definitions since we are only compatible with POSIX systems.
 
 */

#ifndef _HOSTAP_COMPAT_H
#define _HOSTAP_COMPAT_H

#include <sys/time.h>

typedef uint64_t u64;
typedef uint32_t u32;
typedef uint16_t u16;
typedef uint8_t u8;
typedef int64_t s64;
typedef int32_t s32;
typedef int16_t s16;
typedef int8_t s8;

/* Waaad uses the POSIX API, so we don't provide alternatives. This may be changed later as needed */
#define os_malloc(s) malloc((s))
#define os_realloc(p, s) realloc((p), (s))
#define os_free(p) free((p))

#define os_memcpy(d, s, n) memcpy((d), (s), (n))
#define os_memmove(d, s, n) memmove((d), (s), (n))
#define os_memset(s, c, n) memset(s, c, n)
#define os_memcmp(s1, s2, n) memcmp((s1), (s2), (n))

#define os_strdup(s) strdup(s)
#define os_strlen(s) strlen(s)
#define os_strcasecmp(s1, s2) strcasecmp((s1), (s2))
#define os_strncasecmp(s1, s2, n) strncasecmp((s1), (s2), (n))
#define os_strchr(s, c) strchr((s), (c))
#define os_strcmp(s1, s2) strcmp((s1), (s2))
#define os_strncmp(s1, s2, n) strncmp((s1), (s2), (n))
#define os_strncpy(d, s, n) strncpy((d), (s), (n))
#define os_strrchr(s, c) strrchr((s), (c))
#define os_strstr(h, n) strstr((h), (n))
#define os_snprintf snprintf

#define os_random() random()

static __inline__ void * os_zalloc(size_t size)
{
	void *n = os_malloc(size);
	if (n)
		os_memset(n, 0, size);
	return n;
}

typedef long os_time_t;
struct os_time {
	os_time_t sec;
	os_time_t usec;
};

static __inline__ int os_get_time(struct os_time *t)
{
	int res;
	struct timeval tv;
	res = gettimeofday(&tv, NULL);
	t->sec = tv.tv_sec;
	t->usec = tv.tv_usec;
	return res;
}


#ifdef __GNUC__
#define STRUCT_PACKED __attribute__ ((packed))
#else
#define STRUCT_PACKED
#endif


#define INTERNAL_MD5
#define CONFIG_CRYPTO_INTERNAL


#endif /* _HOSTAP_COMPAT_H */
