/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Header file for the sec_tls_gnutls extension. 
 *
 *  This extension provides the TLS mechanism to the waaad daemon.
 *
 * It relies on the gnutls library to perform the actual cryptographic operations.
 */
 
#define IN_EXTENSION
#define DEFINE_DEBUG_MACRO	sec_gnutls
#include <waaad/waaad.h>

#include <string.h>

#include "gnutls_sctp_wrapper.h"

/* The value of the Inband-Security-Id that this extension provides */
#define SEC_TLS_INBAND_SECURITY_ID	1

/**************************************************************************
 * The functions that are provided back to the daemon, as a security module. 
 * See <waaad/security-api.h> 
 **************************************************************************/
int tls_state_change (sec_pss_t newstate, sec_pss_t oldstate, sec_session_t * session, void ** ext_session);
int tls_send_protect (sec_session_t * session, void ** ext_session, void * data, size_t length);
int tls_recv_unprotect (sec_session_t * session, void ** ext_session, void ** data, size_t *length);
int tls_is_supported_peer (char * diamid, struct sockaddr * sa, int * priority);

/**************************************************************************
 * The functions internal to the extension. 
 **************************************************************************/
/* Initialize the extension */
int tls_init(void);

/* Destroy the table and free all data */
int tls_fini(void);

/* verbosity configuration */
extern int sec_gnutls_verbosity;
